/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.util.Iterator;

import midnight.Character;
import midnight.Location;
import midnight.Map;

/**
 */
public class CharacterSelectionScreen extends GameScreen {
	protected static final String selectionKeys = "cvbn1234567890qwertyuiopasdfghjk";
	
	/** the screen header area */
	private TextBox header;
	
	/** Character names in two columns */
	private TextBox[] column;
	
	protected CharacterSelectionScreen(MidnightApplet applet) {
		super(applet);
		
		header = new TextBox(1, 1, 30, 1, applet.font);
		header.setText(" Press a key to choose....");
		column = new TextBox[2];
		column[0] = new TextBox(1, 3, 15, 20, applet.font);
		column[1] = new TextBox(17, 3, 15, 20, applet.font);
	}
	
	public void paint(Graphics g) {
		clearScreen(ig);
		ig.setColor(Color.BLACK);
		
		column[0].setText("");
		column[1].setText("");
		TextBox text = column[0];

		int i = 0;
		int recruited = 0;
		for (Iterator it = getGame().characters(); it.hasNext(); i++) {
			Character c = (Character) it.next();
			if (c.isRecruited() && getGame().isControllable(c)) {
				char selectionChar;
				if (c.isAlive()) {
					if (c.getTime().isNight()) {
						selectionChar = 0x7c;
					}
					else if (c.getEnergy() == 0) {
						selectionChar = 0x7f;
					}
					else {
						selectionChar = 0x23;
					}
				}
				else {
					selectionChar = 0x7d;
				}
				text.append("\n" + selectionChar + "" + selectionKeys.charAt(i) + " " + c.getName());
				recruited++;
				if (recruited % 20 == 0) {
					text = column[1];
				}
			}
		}
		header.paint(ig);
		column[0].paint(ig);
		column[1].paint(ig);
		g.drawImage(screen, 0, 0, getWidth(), getHeight(), null);
	}
	
	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();
		if (key == KeyEvent.VK_ESCAPE) {
			applet.mainScreen();
			return;
		}

		int selection = selectionKeys.indexOf(key);
		int i = 0;
		if (selection != -1) {
			for (Iterator it = getGame().characters(); it.hasNext(); i++) {
				Character c = (Character) it.next();
				if (i == selection && getGame().isControllable(c)) {
					applet.setCharacter(c);
					return;
				}
			}
		}
	}
}