/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.util.HashSet;
import java.util.Iterator;

import midnight.Army;
import midnight.Character;
import midnight.Location;
import midnight.Map;
import midnight.Race;
import midnight.Location.Object;

/**
 * Screen for selecting character actions (seek, hide, etc.).
 */
public class ChooseActionScreen extends GameScreen {
	/** valid keys to enter */
	private HashSet validChoices;

	private Army guards;

	private Character recruiting;

	private Character c;

	private Location.Object object;
	
	/** Area for screen's text */
	private TextBox text;
	
	protected ChooseActionScreen(MidnightApplet applet) {
		super(applet);
		validChoices = new HashSet();
		
		text = new TextBox(1, 1, 30, 30, applet.font);
	}
	
	public void paint(Graphics g) {
		c = applet.character;
		Location inFront = getMap().getInFront(c.getLocation(), c.getDirection());
		
		clearScreen(ig);
		validChoices.clear();
		ig.setColor(Color.BLACK);
		
		text.setText(" Press a key to choose....\n\n");
				
		if (!c.isHidden()) {
			text.append("\n1. Seek");
			validChoices.add(new Integer(1));

			if (c.canHide()) {
				text.append("\n2. Hide");
				validChoices.add(new Integer(2));
			}
		}
		else {
			text.append("\n2. Do not hide");
			validChoices.add(new Integer(2));
		}
		
		object = c.getLocation().getObject();
		if (c.canFight()) {
			text.append("\n3. Fight the " + object.toString(c.getLocation()));
			validChoices.add(new Integer(3));
		}
			
		for (Iterator it = c.getLocation().getCharacters().iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			if (character == c) continue;
			
			/*
			 * Original LOM limitation: if more than one character in location,
			 * can be recruited, will show the option only for the first one.
			 * It's never an issue in the game, though.
			 */
			if (!character.isRecruited() && c.canRecruit(character)) {
				text.append("\n4. Recruit " + character.getFullTitle());
				validChoices.add(new Integer(4));
				recruiting = character;
				break;
			}
		}

		if (c.canRecruitMen()) {
			text.append("\n5. Recruit men");
			validChoices.add(new Integer(5));
		}
		if (c.canStandOnGuard()) {
			text.append("\n6. Stand on guard");
			validChoices.add(new Integer(6));
		}
				
		/*
		 * LOM code also checks that FreeArmyPosInTable is >= 0x1d (what is it?)
		 */
		if (c.canAttack()) {
			text.append("\n7. To battle!");
			validChoices.add(new Integer(7));
		}
		
		text.paint(ig);
		g.drawImage(screen, 0, 0, getWidth(), getHeight(), null);
	}

	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();
		switch (key) {
			case KeyEvent.VK_ESCAPE:
			case 'e':
				applet.mainScreen();
				return;
			case KeyEvent.VK_ENTER:
			case 'r':
				applet.think();
				return;
		}
				
		if (key < '1' || key > '9') return;
		int choice = key - '0';
		if (!validChoices.contains(new Integer(choice))) return;
		
		Graphics g = e.getComponent().getGraphics();
		
		switch (choice) {
			case 1:
				applet.characterSeek();
				break;
			case 2:
				applet.characterHide();
				break;
			case 3:
				applet.characterFight();
				break;
			case 4:
				applet.characterRecruit(recruiting);
				return;
			case 5:
				applet.characterRecruitMen();
				break;
			case 6:
				applet.characterStandOnGuard();
				break;
			case 7:
				applet.characterAttack();
				break;
		}
	}
}
