/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;

import midnight.Map;
import midnight.Midnight;

/**
 * Interface representing a screen in the game.
 */
public abstract class GameScreen extends Component implements KeyListener{
	/** The applet in which we're running */
	protected MidnightApplet applet;
	
	/** Off-screen image buffer */
	protected BufferedImage screen;
	
	/** Image buffer's graphic context */
	protected Graphics ig;
	
	protected GameScreen(MidnightApplet applet) {
		this.applet = applet;

		screen = new BufferedImage(256, 192, BufferedImage.TYPE_INT_ARGB);
		ig = screen.getGraphics();
	}
	
	/**
	 * Returns game instance.
	 */
	protected Midnight getGame() {
		return applet.game;
	}
	
	/**
	 * Returns map of the instance.
	 */
	protected Map getMap() {
		return applet.map;
	}
	
	/**
	 * @see java.awt.Component#paint
	 */
	public abstract void paint(Graphics g);
	
	/** 
	 * Clears the screen with white background.
	 */
	protected void clearScreen(Graphics g) {
		clearScreen(g, Color.WHITE);
	}
	
	/**
	 * Clears the screen with the given background colour.
	 */
	protected void clearScreen(Graphics g, Color background) {
		Color color = g.getColor();
		g.setColor(background);
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setColor(color);
	}
	
	/**
	 * @see java.awt.event.KeyListener#keyPressed(KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
	}

	/**
	 * @see java.awt.event.KeyListener#keyReleased(KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
	}

	/**
	 * @see java.awt.event.KeyListener#keyTyped(KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
	}
	
	/**
	 * Returns the capitalized version of a string.
	 */
	public static StringBuffer capitalize(Object o) {
		StringBuffer sb = new StringBuffer(o.toString());
		sb.setCharAt(0, java.lang.Character.toUpperCase(sb.charAt(0)));
		return sb;
	}
}
