/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.util.Iterator;

import midnight.Army;
import midnight.Character;
import midnight.Location;
import midnight.Map;

/**
 */
public class MainScreen extends GameScreen {
	private boolean isNight;
	
	/** Text area for character's title */
	private TextBox title;
	
	/** Text area for describing character's location */
	private TextBox text;
	
	protected MainScreen(MidnightApplet applet) {
		super(applet);
		title = new TextBox(1, 1, 23, 20, applet.font);
		text = new TextBox(1, 2, 23, 20, applet.font);
	}
	
	public void paint(Graphics g) {
		Character character = applet.character;
		Location location = character.getLocation();
		Location looking = getMap().getLookingTowards(location, character.getDirection());
		isNight = character.getTime().isNight();
		
		drawHorizon(ig);
		
		printWhereHeIs(ig, character, location, looking);
		drawShield(ig, character);
		drawPanoramic(ig, location, character.getDirection(), isNight);
		drawInFront(ig, character);
		g.drawImage(screen, 0, 0, getWidth(), getHeight(), null);
	}

	public void printWhereHeIs(Graphics g,
		Character character, Location location, Location looking) {
		g.setColor(Color.YELLOW);
		
		title.setText(character.getFullTitle().toUpperCase());
		title.paint(g);
		
		StringBuffer whereHeIs = new StringBuffer(" He stands ");
		whereHeIs.append(getPreposition(location.getFeature()));
		whereHeIs.append(" ").append(location);
		whereHeIs.append(" looking ");
		whereHeIs.append(character.getDirection());
		if (location.getDomain() != looking.getDomain() ||
			location.getDomainFlag() != looking.getDomainFlag() ||
			location.getFeature() != looking.getFeature()) {
			
			whereHeIs.append(" to ");
			whereHeIs.append(looking);
		}
		whereHeIs.append(".");
		g.setColor(Color.CYAN);
		text.setText(whereHeIs.toString());
		text.paint(g);
	}

	protected void drawShield(Graphics g, Character c) {
		applet.getShield(c).paint(g, 25, 1);
	}
	
	protected void drawHorizon(Graphics g) {
		clearScreen(g);
		
		// Draw the "sky"
		if (isNight) {
			g.setColor(Color.BLUE);
		}
		else {
			g.setColor(Color.BLUE);
		}
		g.fillRect(0, 0, 256, 112);
		
		if (isNight) {
			g.setColor(Color.BLACK);
			g.fillRect(0, 112, 256, 84);
			g.setColor(Color.BLUE);
			g.drawLine(0, 112, 256, 112);
		}
	}
	
	private String getPreposition(Location.Feature feature) {
		if (feature == feature.MOUNTAIN || feature == feature.FOREST || 
			feature == feature.VILLAGE || feature == feature.FROZEN_WASTE ||
			feature == feature.RUIN || feature == feature.ARMY) {

			return "in";
		}
		else if (feature == feature.CITADEL || feature == feature.HENGE ||
			feature == feature.TOWER || feature == feature.KEEP ||
			feature == feature.SNOWHALL || feature == feature.LITH ||
			feature == feature.CAVERN) {
			
			return "at";
		}
		else if (feature == feature.DOWNS || feature == feature.PLAINS) {
			return "on";
		}
		else {
			return "beside";
		}
	}
	
	int [] column = {12, 20, 16, 4, 8, 24, 0, 28};
	
	/**
	 * Draws the characters, doomdark's armies, and nasties directly in front of the
	 * character.
	 */
	private void drawInFront(Graphics g, Character character) {
		Location location = getMap().getInFront(character.getLocation(), character.getDirection());
		int drawn = 0;
		
		for (Iterator it = location.getCharacters().iterator(); it.hasNext() && drawn < column.length;) {
			Character c = (Character) it.next();
			
			if (c == character) continue;
			
			drawCharacter(g, c, column[drawn++]);
		}
		
		/** Draw riders first, if present */
		for (Iterator it = location.getArmies().iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			if (army.getType() == Army.Type.RIDERS && army.getHowMany() > 0) {
				while (drawn < 8) {
					drawDoomguard(g, army, column[drawn++]);
				}
				return;
			}
		}
		for (Iterator it = location.getArmies().iterator(); it.hasNext();) {
			Army army = (Army) it.next();
			if (army.getType() == Army.Type.WARRIORS && army.getHowMany() > 0) {
				while (drawn < 8) {
					drawDoomguard(g, army, column[drawn++]);
				}
				return;
			}
		}
		
		Location.Object object = location.getObject();
		int count = 0;
		if (object == object.DRAGONS) {
			count = 2;
		}
		else if (object == object.ICE_TROLLS || object == object.SKULKRIN ||
			object == object.WILD_HORSES || object == object.WOLVES) {
			count = 4;
		}
		for (int i = 0; drawn < 8 && i < count; i++) {
			drawLocationObject(g, object, column[drawn++]);
		}
	}
	
	private void drawCharacter(Graphics g, Character character, int column) {
		applet.getEntity(character).paint(g, column, 23, isNight);
	}
	
	private void drawDoomguard(Graphics g, Army army, int column) {
		applet.getEntity(army).paint(g, column, 23, isNight);
	}
	
	private void drawLocationObject(Graphics g, Location.Object object, int column) {
		//midnight.ui.applet.LocationObject.get(object).paint(g, column*8 - 4, 23*8);
		applet.getEntity(object).paint(g, column, 23, isNight);
	}
	
	private void drawPanoramic(Graphics g, Location location, Map.Direction direction, boolean isNight) {
		int howLeftScr;
		int howRightScr;
		
		if (direction.isDiagonal()) {
			howLeftScr = 2;
			howRightScr = 3;
		}
		else {
			howLeftScr = 1;
			howRightScr =0;
		}
		
		for (int i = 0; i < 31; i++) {
			drawLocation(g, location, landscape[i], direction, howLeftScr, isNight);
			drawLocation(g, location, landscape[i], direction.turnRight(), howRightScr, isNight);
		}
	}
	
	private void drawLocation(Graphics g, Location from, Landscape landscape, Map.Direction direction, int scrType, boolean isNight) {
		Location location = calculateLocation(from, landscape, direction);
//		System.err.println("Draw " + location);
		int x = 320;
		switch (scrType) {
			case 0:
				x = x + landscape.xadj;
				break;
			case 1:
				x = x - landscape.xadj;
				break;
			case 2:
				x = x + landscape.xadj - 100;
				break;
			case 3:
				x = x - landscape.xadj + 100;
				break;
		}
		
		drawFeature(g, location.getFeature(), x, landscape.y, landscape.size, isNight);
	}
	
	/**
	 * Draws a feature on the screen.
	 * 
	 * @param feature feature to draw
	 * @param x screen X coordinate
	 * @param y screen Y coordinate
	 * @param size feature size
	 */
	private void drawFeature(Graphics g, Location.Feature feature, int x, int y, int size, boolean isNight) {
//		System.err.println("  " + feature + " (" + x + ", " + y + ") - " + size);
		if (feature == feature.PLAINS) return;
		Feature.get(feature, size).paint(g, x, y, isNight);
	}
	
	/**
	 * Uses landscaping coordinate manipulation constants to calculate locations that need to be
	 * drawn, where they should be on the screen, and in which size.
	 */
	private Location calculateLocation(Location from, Landscape landscape, Map.Direction direction) {
		int x = from.getX();
		int y = from.getY();
		
		if (direction == direction.NORTH) {
			x -= landscape.dx;
			y -= landscape.dy;
		}
		else if (direction == direction.NORTHEAST) {
			x += landscape.dx;
			y -= landscape.dy;
		}
		else if (direction == direction.EAST) {
			x += landscape.dy;
			y -= landscape.dx;
		}
		else if (direction == direction.SOUTHEAST) {
			x += landscape.dy;
			y += landscape.dx;
		}
		else if (direction == direction.SOUTH) {
			x += landscape.dx;
			y += landscape.dy;
		}
		else if (direction == direction.SOUTHWEST) {
			x -= landscape.dx;
			y += landscape.dy;
		}
		else if (direction == direction.WEST) {
			x -= landscape.dy;
			y += landscape.dx;
		}
		else if (direction == direction.NORTHWEST) {
			x -= landscape.dy;
			y -= landscape.dx;
		}
		
		return getMap().getLocation(x, y);
	}
	
	private static final Landscape landscape [] = {
		new Landscape(5, 4,  86, 64, 7),
		new Landscape(4, 5, 114, 64, 7),
		new Landscape(6, 2,  41, 64, 7),
		new Landscape(6, 1,  21, 64, 7),
		new Landscape(6,-1, -21, 64, 7),
		new Landscape(6, 0,   0, 64, 7),
		new Landscape(5, 3,  69, 63, 7),
		new Landscape(3, 5, 131, 63, 7),
		new Landscape(4, 4, 100, 63, 7),
		new Landscape(5, 2,  48, 63, 6),
		new Landscape(5, 1,  25, 63, 6),
		new Landscape(5,-1, -25, 63, 6),
		new Landscape(5, 0,   0, 62, 6),
		new Landscape(4, 3,  82, 62, 6),
		new Landscape(3, 4, 118, 62, 6),
		new Landscape(4, 2,  59, 62, 6),
		new Landscape(3, 3, 100, 61, 5),
		new Landscape(4, 1,  31, 61, 5),
		new Landscape(4,-1, -31, 61, 5),
		new Landscape(4, 0,   0, 60, 5),
		new Landscape(3, 2,  75, 59, 5),
		new Landscape(2, 3, 125, 59, 5),
		new Landscape(3, 1,  41, 58, 4),
		new Landscape(3,-1, -41, 58, 4),
		new Landscape(3, 0,   0, 57, 4),
		new Landscape(2, 2, 100, 57, 4),
		new Landscape(2, 1,  59, 53, 3),
		new Landscape(1, 2, 141, 53, 3),
		new Landscape(2, 0,   0, 51, 2),
		new Landscape(1, 1, 100, 43, 1),
		new Landscape(1, 0,   0, 32, 0),
	};
	
	/**
	 * Landscaping calculation table.
	 */
	private static class Landscape {
		/** location y-coordinate adjuster */
		protected short dy;
		
		/** location x-coordinate adjuster */
		protected short dx;
		
		/** screen x adjuster */
		protected short xadj;
		
		/** screen y position */
		protected short y;
		
		/** scale */
		protected short size;
		
		protected Landscape(int dy, int dx, int xadj, int y, int size) {
			this.dy = (short) dy;
			this.dx = (short) dx;
			this.xadj = (short) xadj;
			this.y = (short) y;
			this.size = (short) size;
		}
	}
	
	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();
		Character character = applet.character;

		switch (key) {
			case 'q':
			case '0':
				if (character.canWalkForward()) {
					applet.walk();
				}
				else {
					applet.think();
				}
				break;
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
				character.setDirection(Map.Direction.getDirection(key - '1'));
				repaint();
				break;
			case '[':
				character.setDirection(character.getDirection().turnLeft());
				repaint();
				break;
			case ']':
				character.setDirection(character.getDirection().turnRight());
				repaint();
				break;
			case 'c':
				applet.setCharacter(getGame().LUXOR);
				break;
			case 'v':
				applet.setCharacter(getGame().MORKIN);
				break;
			case 'b':
				applet.setCharacter(applet.game.CORLETH);
				break;
			case 'n':
				applet.setCharacter(applet.game.ROTHRON);
				break;
			case 'm':
				applet.chooseCharacter();
				break;
			case 't':
				applet.chooseAction();
				break;
			case 'r':
				applet.think();
				break;
			case 'u':
				applet.night();
				break;
			case '=':
				applet.viewMap();
				break;
			case 's':
				applet.save();
				break;
			case 'l':
				applet.load();
				break;
		}
	}
}
