/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.util.Iterator;

import midnight.Army;
import midnight.Character;
import midnight.Location;
import midnight.Race;

/**
 */
public class MapScreen extends GameScreen implements MouseListener {
	private static int[] PLAINS = {136, 212, 92};
	private static int[] FOREST = {40, 124, 0};
	private static int[] DOWNS = {232, 212, 80};
	private static int[] MOUNTAINS = {216, 116, 8};
	private static int[] LAKE = {0, 0, 255};
	private static int[] KEEP = {0, 0, 0};
	private static int[] FREE = {255, 255, 255};
	private static int[] FOUL = {255, 0, 0};
	private static int[] NASTY = {255, 128, 0};
	private static int[] WASTES = {200, 200, 255};
	
	private int x = -1;
	private int y = -1;
	
	/** Area to display selected location's name */
	private TextBox location;
	
	/** Area to display selected location's object */
	private TextBox object;
	
	/** Area to display selected location's guard */
	private TextBox guard;
	
	/** Area to display selected location's characters */
	private TextBox characters;
	
	BufferedImage map;
	/**
	 * Constructor for MapScreen.
	 * @param applet
	 */
	public MapScreen(MidnightApplet applet) {
		super(applet);
		int width = getMap().width() * 3;
		int height = getMap().height() * 3;
		map = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		
		location = new TextBox(0, 0, 48, 2, applet.font);
		object = new TextBox(48, 0, 30, 1, applet.font);
		guard = new TextBox(48, 2, 30, 4, applet.font);
		characters = new TextBox(48, 6, 30, 30, applet.font);
	}

	/**
	 * @see java.awt.Component#paint(Graphics)
	 */
	public void paint(Graphics g) {
		clearScreen(g);
		WritableRaster raster = map.getRaster();
		for (int y = 0; y < getMap().height(); y++) {
			for (int x = 0; x < getMap().width(); x++) {
				Location location = getMap().getLocation(x, y);
				Location.Feature feature = location.getFeature();
				int [] pixel;
				if (feature == feature.LAKE) {
					pixel = LAKE;
				}
				else if (feature == feature.DOWNS) {
					pixel = DOWNS;
				}
				else if (feature == feature.MOUNTAIN) {
					pixel = MOUNTAINS;
				}
				else if (feature == feature.FOREST) {
					pixel = FOREST;
				}
				else if (feature == feature.KEEP || feature == feature.CITADEL) {
					pixel = KEEP;
				}
				else if (feature == feature.FROZEN_WASTE) {
					pixel = WASTES;
				}
				else {
					pixel = PLAINS;
				}
				for (int tileY = 0; tileY < 3; tileY++) {
					for (int tileX = 0; tileX < 3; tileX++) {
						raster.setPixel(3*x + tileX, 3*y + tileY, pixel);
					}
				}

				if (pixel == PLAINS && (feature != feature.PLAINS && feature != feature.ARMY)) {
					raster.setPixel(3*x + 1, 3*y + 1, KEEP);
				}
				else if (pixel == KEEP) {
					if (location.getGuard().getRace() == Race.FOUL) {
						raster.setPixel(3*x+1, 3*y+1, FOUL);
					}
					else {
						raster.setPixel(3*x+1, 3*y+1, FREE);
					}
				}
				if (location.getObject() == Location.Object.WOLVES || 
					location.getObject() == Location.Object.DRAGONS ||
					location.getObject() == Location.Object.ICE_TROLLS ||
					location.getObject() == Location.Object.SKULKRIN) {
						
					raster.setPixel(3*x+1, 3*y+1, FOUL);
				}
				if (location.getObject() == Location.Object.WILD_HORSES) {
						
					raster.setPixel(3*x+1, 3*y+1, FOREST);
				}
				
				if (!location.getCharacters().isEmpty()) {
					for (int tileY = 0; tileY < 3; tileY++) {
						for (int tileX = 0; tileX < 3; tileX++) {
							raster.setPixel(3*x + tileX, 3*y + tileY, FREE);
						}
					}
					if (!location.getArmies().isEmpty()) {
						raster.setPixel(3*x+1, 3*y+1, FOUL);
					}
				}
				else if (!location.getArmies().isEmpty()) {
					for (int tileY = 0; tileY < 3; tileY++) {
						for (int tileX = 0; tileX < 3; tileX++) {
							raster.setPixel(3*x + tileX, 3*y + tileY, FOUL);
						}
					}
				}
			}
		}
		g.drawImage(map, 0, getHeight() - 2*map.getHeight(), map.getWidth()*2, map.getHeight()*2, null);
		
		if (x >= 0 && y >= 0) {
			Location location = getMap().getLocation(x, y);
			this.location.setText(location.toString());
			this.location.paint(g);

			Location.Object object = location.getObject();
			String description;
			if (object == object.CUP_OF_DREAMS) {
				description = "cup";
			}
			else if (object == object.DRAGONS) {
				description = "dragons";
			}
			else if (object == object.DRAGONSLAYER) {
				description = "dragonslayer";
			}
			else if (object == object.GUIDANCE) {
				description = "guidance";
			}
			else if (object == object.HAND_OF_DARK) {
				description = "darkness";
			}
			else if (object == object.ICE_CROWN) {
				description = "Ice Crown";
			}
			else if (object == object.ICE_TROLLS) {
				description = "trolls";
			}
			else if (object == object.MOON_RING) {
				description = "Moon Ring";
			}
			else if (object == object.SHADOWS_OF_DEATH) {
				description = "shadows";
			}
			else if (object == object.SHELTER) {
				description = "shelter";
			}
			else if (object == object.SKULKRIN) {
				description = "skulkrin";
			}
			else if (object == object.WATERS_OF_LIFE) {
				description = "waters";
			}
			else if (object == object.WILD_HORSES) {
				description = "horses";
			}
			else if (object == object.WOLFSLAYER) {
				description = "wolfslayer";
			}
			else if (object == object.WOLVES) {
				description = "wolves";
			}
			else  {
				description = "";
			}
			this.object.setText(description);
			this.object.paint(g);
			
			if (location.getGuard() != null) {
				Army guard = location.getGuard();
				this.guard.setText(guard.getRace().toString());
				this.guard.append("\nG: " + guard.getHowMany() + (guard.getType() == Army.Type.WARRIORS ? "W" : "R"));
			}
			else {
				this.guard.setText("");
			}
			int warriors = 0;
			int riders = 0;
			for (Iterator it = location.getArmies().iterator(); it.hasNext();) {
				Army army = (Army) it.next();
				if (army.getType() == Army.Type.WARRIORS) {
					warriors += army.getHowMany();
				}
				else {
					riders += army.getHowMany();
				}
			}
			if (warriors > 0) {
				this.guard.append("\nW: " + warriors);
			}
			if (riders > 0) {
				this.guard.append("\nR: " + riders);
			}
			this.guard.paint(g);
			
			characters.setText("");
			for (Iterator it = location.getCharacters().iterator(); it.hasNext();) {
				Character character = (Character) it.next();
				characters.append("\n" + character.getName());
				warriors = character.getWarriors().getHowMany();
				riders = character.getRiders().getHowMany();
				if (warriors > 0) {
					characters.append("\n  W: " + warriors);
				}
				if (riders > 0) {
					characters.append("\n  R: " + riders);
				}
			}
			characters.paint(g);
		}
	}
	
	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();

		switch (key) {
			case KeyEvent.VK_ESCAPE:
			case '=':
			case 'e':
				removeMouseListener(this);
				x = y = -1;
				applet.mainScreen();
				return;
			case 'm':
				removeMouseListener(this);
				x = y = -1;
				applet.chooseCharacter();
				return;
			case 't':
				removeMouseListener(this);
				x = y = -1;
				applet.chooseAction();
				return;
			case 'r':
				removeMouseListener(this);
				x = y = -1;
				applet.think();
				return;
		}
	}
	/**
	 * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		int x = e.getX();
		int y = e.getY() - (getHeight() - map.getHeight()*2);
		if (x > map.getWidth()*2 || y > map.getHeight()*2) return;
		this.x = x/6;
		this.y = y/6;
		repaint();
	}

	/**
	 * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
	}

	/**
	 * @see java.awt.event.MouseListener#mouseExited(MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
	}

	/**
	 * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
	}

	/**
	 * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
	}
}
