/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.util.Iterator;

import midnight.Map;
import midnight.Race;

/**
 * Screen for ending the turn and conducting (and reporting) on night
 * time activities.
 */
public class NightScreen extends GameScreen {
	private boolean dawn;

	/** Area for screen's text */
	private TextBox text;
	
	/**
	 * Constructor.
	 */
	public NightScreen(MidnightApplet applet) {
		super(applet);
		
		text = new TextBox(1, 1, 30, 20, applet.font);
	}

	/**
	 * @see java.awt.Component#paint(Graphics)
	 */
	public void paint(Graphics g) {
		clearScreen(ig, Color.BLACK);
		ig.setColor(Color.YELLOW);
		
		StringBuffer message = new StringBuffer("Night has fallen and the Foul are abroad! ");
		int day = applet.game.getDay();
		message.append(day);
		message.append(" day");
		if (day == 1) {
			message.append(" has");
		}
		else {
			message.append("s have");
		}
		message.append(" passed since the War of the Solstice began. ");
		
		message.append(getGameStatus());
		
		text.setText(message.toString());
		text.paint(ig);
		g.drawImage(screen, 0, 0, getWidth(), getHeight(), null);
	}

	protected String getGameStatus() {
		StringBuffer status = new StringBuffer();
		if (applet.game.isGameOver()) {
			status.append(applet.game.getStatus());
			if (applet.game.getStatus().getWinner() == Race.FOUL) {
				status.append(" Victory to Doomdark.");
			}
			else {
				status.append(" Victory to the Free!");
			}
		}
		else {
			boolean first = true;
			for (Iterator it = applet.game.getBattleDomains().iterator(); it.hasNext();) {
				Map.Area domain = (Map.Area) it.next();
				if (first) {
					status.append("\n  The bloody sword of battle brings death in the Domain");
					first = false;
				}
				else {
					status.append(" and");
				}
				status.append(" of ");
				status.append(domain);
			}
			if (!first) {
				status.append(".");
			}
		}
		status.append("\n  Do you want dawn?");
		
		return status.toString();
	}

	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();
		switch (key) {
			case KeyEvent.VK_ENTER:
			case 'g':
				dawn = true;
				applet.dawn();
				break;
		}
	}
}
