/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Class that's used to draw a character's shield.
 */
public class Shield {
	/** Game applet */
	private MidnightApplet applet;

	/** parts making up the shield */
	short [] shieldData;
	
	Shield(MidnightApplet applet, short [] shieldParts) {
		this.applet = applet;
		this.shieldData = shieldParts;
	}

	/**
	 * Draws the appropriate image of the shield at the given row and column. The image is made 
	 * up of a varying number of coloured standard parts, where each part consists of a grid of 
	 * 8x8 tiles represented as bitmaps in the <code>MidnightApplet.shieldFont</code> "font".
	 */
	public void paint(Graphics g, int column, int row) {
		int shieldPaper = shieldData[0]*8;
		int shieldInk = 1;
		byte attribute = (byte)(shieldPaper | shieldInk);
		drawShieldPart(g, column, row, (short)0, attribute);
		int i = 1;
		while (i < shieldData.length) {
			short part = (short) (shieldData[i] & 0x07);
			shieldInk = shieldData[i] >> 3;
			attribute = (byte)(shieldPaper | shieldInk);
			i++;
			
			int dx = shieldData[i] & 0x07;
			int dy = shieldData[i] >> 3;
			i++;
			
			drawShieldPart(g, column + dx, row + dy, part, attribute);
		}
	}
	
	/**
	 * 
	 */
	protected void drawShieldPart(Graphics g, int column, int row, short part, byte attribute) {
		attribute |= 0x40;
		
		short [] partData = applet.getShieldPartData(part);
		int i = 0;
		int width = partData[i++];
		int height = partData[i++];
		for (int dy = 0; dy < height; dy++) {
			for (int dx = 0; dx < width; dx++) {
				short ch = partData[i++];
				if (ch != 0) {
					drawChar(g, (column + dx) * 8, (row+dy) * 8, ch, attribute);
				}
			}
		}
	}
	
	/**
	 * Draws a single 8x8 character from <code>MidnightApplet.shieldFont</code> font
	 * in the given colour at a specified (x, y) coordinate.
	 * 
	 * an attribute byte is broken down as follows
	 * 7		6		5		4		3		2		1		0
	 * --------------------------------------------------------------
	 * F		B		PAP_G	PAP_R	PAP_B	INK_G	INK_R	INK_B
	 *
	 * where the 3bits for INK and PAPER are RGB values
	 * B is the BRIGHT bit, if it is a '1' then the colours (except black) appear brighter
	 * F is the FLASH bit, if it is a '1' then the ULA swaps the PAPER and INK values.
	 * 
	 * Colours are:
	 * 	0 - black
	 *  1 - blue
	 *  2 - red
	 *  3 - magenta
	 *  4 - green
	 *  5 - cyan
	 *  6 - yellow
	 *  7 - white
	 */
	protected void drawChar(Graphics g, int x, int y, short ch, byte attribute) {
		Color fg = color[attribute & 0x07];
		Color bg = color[(attribute >> 3) & 0x07];
		for (int dy = 0; dy < 8; dy++, y++) {
			byte data = applet.shieldFont[ch][dy];
			for (int dx = 7; dx >= 0; dx--, x++) {
				if ((data & (1<<dx)) == 0) {
					g.setColor(fg);
				}
				else {
					g.setColor(bg);
				}
				g.drawLine(x, y, x, y);
			}
			x -= 8;
		}
	}
	
	/** Mapping from Spectrum's to AWT colours */
	private static final Color [] color = {
		Color.BLACK, Color.BLUE, Color.RED, Color.MAGENTA,
		Color.GREEN, Color.CYAN, Color.YELLOW, Color.WHITE,
	};
}
