/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Graphics;

/**
 * A multi-line read-only area that displays text.
 */
public class TextBox {
	/** contents of the text box */
	protected StringBuffer text;
	
	/** x coordinate of the top-left corner */
	protected int xOrigin;
	/** y coordinate of the top corner */
	protected int yOrigin;
	/** width of the box in characters */
	protected int width;
	/** height of the box in characters */
	protected int height;
	
	/** font to use */
	protected byte[][] font;
	
	/**
	 * Constructor.
	 */
	public TextBox(int x, int y, int width, int height, byte [][] font) {
		this.xOrigin = x;
		this.yOrigin = y;
		this.width = width;
		this.height = height;
		this.font = font;
	}

	/**
	 * Sets the box's contents.
	 */
	public void setText(String text) {
		this.text = new StringBuffer(text);
	}
	
	/**
	 * Appends to the box's contents.
	 */
	public void append(String text) {
		this.text.append(text);
	}
	
	public void paint(Graphics g) {
		//char [] msg = text.toCharArray();
		int curWidth = 0;
		int x = xOrigin;
		int y = yOrigin;
		StringBuffer curLine = new StringBuffer();
		for (int i = 0; i < text.length() && y < yOrigin+height; i++) {
			char ch = text.charAt(i);
			if (ch == '\n') {
				drawString(g, trimRight(curLine).toString(), x, y);
				curLine.setLength(0);
				curWidth = 0;
				y++;
				continue;
			}
			if (curWidth  > width) {
				int j = curLine.lastIndexOf(" ");
				drawString(g, curLine.substring(0, j+1), x, y);
				curLine.delete(0, j+1);
				curWidth = curLine.toString().length();
				y++;
			}
			curLine.append(ch);
			curWidth++;
		}
		if (y < yOrigin+height) {
			drawString(g, curLine.toString(), x, y);
		}
	}

	/**
	 * Draws the text given by the specified string, using the
	 * applet's font as defined by glyphs in <code>applet.font</code>.
	 */
	protected void drawString(Graphics g, String s, int x, int y) {
		x *= 8;
		y *= 8;
		for (int i = 0; i < s.length(); i++) {
			char ch = s.charAt(i);
			for (int dy = 0; dy < 8; dy++, y++) {
				byte data = font[ch-' '][dy];
				for (int dx = 7; dx >= 0; dx--, x++) {
					if ((data & (1<<dx)) != 0) {
						g.drawLine(x, y, x, y);
					}
				}
				x -= 8;
			}
			y -= 8;
			x+= 8;
		}
	}
	
	/**
	 * Removes trailing whitespace.
	 * 
	 * @param sb StringBuffer to trim
	 * @return trimmed argument
	 */
	protected static StringBuffer trimRight(StringBuffer sb) {
		for (int i = sb.length() - 1; i >= 0; i--) {
			char ch = sb.charAt(i);
			if (java.lang.Character.isWhitespace(ch)) {
				sb.deleteCharAt(i);
			}
			else return sb;
		}
		return sb;
	}
}
