/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.ui.applet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.util.Iterator;

import midnight.Army;
import midnight.Location;

/**
 */
public class ThinkScreen extends GameScreen {
	/** Current step in the "think again" sequence */
	private int step;
	
	/** Total number of steps in the sequence */
	private int maxStep;

	/** description of the object the character found (to avoid getting different GUIDANCE on every repaint) */
	private String foundObjectDescription;

	/** for which character the foundObjectDescription is valid */
	private midnight.Character foundObjectCharacter;
	
	/** at which location was the object found */
	private Location foundObjectLocation;

	/** object that was found */
	private Location.Object foundObject;
	
	/** Area for character's title */
	private TextBox title;
	
	/** Area for screen's text */
	private TextBox text;

	protected ThinkScreen(MidnightApplet applet) {
		super(applet);
		
		title = new TextBox(1, 1, 30, 1, applet.font);
		text = new TextBox(1, 11, 30, 10, applet.font);
	}
		
	public void paint(Graphics g) {
		midnight.Character c = applet.character;
		Location location = c.getLocation();
		Location looking = getMap().getLookingTowards(c.getLocation(), c.getDirection());
		Location inFront = getMap().getInFront(c.getLocation(), c.getDirection());
		
		int stage = 0;

		clearScreen(ig);
		
		ig.setColor(Color.RED);
		title.setText(capitalize(c.getFullTitle()).toString());
		title.paint(ig);
		
		applet.getEntity(c).paint(ig, 9, 10);

		Feature f = Feature.get(c.getLocation().getFeature(), 0);
		if (f != null) f.paint(ig, 0x180, 0x60);
		
		ig.setColor(Color.RED);
		
		StringBuffer message = new StringBuffer(" ");
		
		if (step == stage) {
			if (c.isHidden()) {
				message.append(c.getName()).append(" is hidden. ");
			}
			
			if (c.getKilled() != null && c.getKilled() != Location.Object.NOTHING) {
				if (c.isAlive()) {
					message.append("He slew the ").append(c.getKilled().toString(location));
					if (c.getObject() == Location.Object.DRAGONSLAYER ||
						c.getObject() == Location.Object.WOLFSLAYER) {
						message.append(" with ").append(c.getObject().toString(location));
					}
					message.append(". ");
				}
				else {
					message.append("The ").append(c.getKilled().toString(location)).append(" slew him.\n");
				}
			}
	
			if (c.getLocation().getObject() != Location.Object.NOTHING) {
				Location.Object object = c.getLocation().getObject();
				if (object == object.DRAGONS || object == object.ICE_TROLLS || 
					object == object.SKULKRIN || object == object.WOLVES) {
					message.append("The ");
					message.append(object.toString(location));
					message.append(" are abroad and he must fight or hide. ");
				}
				else if (object == object.WILD_HORSES && c.getFound() == null) {
					message.append(Location.capitalize(object.toString(location)));
					message.append(" are abroad. ");
				}
			}
			
			if (c.getFound() != null) {
				if (foundObjectDescription == null || foundObjectCharacter != c ||
					foundObjectLocation != c.getLocation() || foundObject != c.getFound()) {
					foundObjectCharacter = c;
					foundObjectLocation = c.getLocation();
					foundObject = c.getFound();
					foundObjectDescription = foundObject.toString(foundObjectLocation);
				}
				message.append("He has found ");
				message.append(foundObjectDescription);
				message.append(". ");
			}
			else {
				foundObjectDescription = null;
			}
			
			message.append(c.getTime().toString());
			message.append(" and ").append(c.getName());
			message.append(" is ").append(c.getCondition()).append(".");
			
			message.append(" The Ice Fear is ").append(c.getLocation().describeIceFear()).append(". ");
			message.append(c.getName()).append(" is ").append(c.getCourage()).append(".");
			
			if (c.getObject() != Location.Object.NOTHING) {
				message.append(" He has with him the ").append(c.getObject().toString(location)).append(".");
			}
		}
		stage++;		
		
		if (c.getBattle() != null) {
			if (stage == step) {
				if (!c.isAlive()) {
					message.append("He was killed in battle. ");
				}
				message.append(c.describeBattle());
			}
			stage++;
		}

		if ((inFront.getGuard() != null && inFront.getGuard().getHowMany() > 0) || 
			(inFront.getArmies() != null && !inFront.getArmies().isEmpty())) {
			if (stage == step) {
				message.append("To the ").append(c.getDirection()).append(", ");
				if (inFront.getArmies() != null && !inFront.getArmies().isEmpty()) {
					int riders = 0;
					int warriors = 0;
					for (Iterator it = inFront.getArmies().iterator(); it.hasNext();) {
						Army army = (Army) it.next();
						
						if (army.getType() == Army.Type.RIDERS) {
							riders += army.getHowMany();
						}
						else {
							warriors += army.getHowMany();
						}
					}
					message.append("Doomdark has an army of ");
					if (warriors != 0) {
						message.append(warriors);
					}
					else {
						message.append("no");
					}
					message.append(" warriors and ");
	
					if (riders != 0) {
						message.append(riders);
					}
					else {
						message.append("no");
					}
					message.append(" riders. ");
				}

				Army guard = inFront.getGuard();
				if (guard != null && guard.getHowMany() > 0) {
					message.append(guard + " of the " + guard.getRace() + " guard the ");
					message.append(inFront.getFeature());
					message.append('.');
				}
			}
			stage++;
		}
		
		Army guard = location.getGuard();
		if (guard != null && guard.getHowMany() > 0) {
			if (stage == step) {
				message.append(guard + " of the " + guard.getRace() + " guard the ");
				message.append(location.getFeature());
				message.append('.');
				//return;
			}
			stage++;
		}
				
		if (inFront.getCharacters() != null && !inFront.getCharacters().isEmpty()) {
			for (Iterator it = inFront.getCharacters().iterator(); it.hasNext();) {
				midnight.Character character = (midnight.Character) it.next();
				if (stage == step) {
					message.append("To the ").append(c.getDirection()).append(", ");
					message.append(capitalize(character.getFullTitle())).append(" commands ");
					Army warriors = character.getWarriors();
					Army riders = character.getRiders();
					
					message.append(warriors.toString());
					if ((warriors.getHowMany() == 0 && riders.getHowMany() == 0) ||
						(warriors.getHowMany() !=0 && riders.getHowMany() != 0)) {
						message.append(" and ");
					}
					else {
						message.append(" but ");
					}
					message.append(riders.toString());
					message.append(".");
					
					if (warriors != null && warriors.getHowMany() != 0) {
						message.append(" His warriors are ").append(warriors.getCondition()).append(". ");
					}
					if (riders != null && riders.getHowMany() != 0) {
						message.append(" His riders are ").append(riders.getCondition()).append(".");
					}

					//return;
				}
				stage++;
			}
		}

		if (location.getArmies() != null && !location.getArmies().isEmpty()) {
			if (stage == step) {
				int riders = 0;
				int warriors = 0;
				for (Iterator it = location.getArmies().iterator(); it.hasNext();) {
					Army army = (Army) it.next();
					
					if (army.getType() == Army.Type.RIDERS) {
						riders += army.getHowMany();
					}
					else {
						warriors += army.getHowMany();
					}
				}
				message.append("Doomdark has an army of ");
				if (warriors != 0) {
					message.append(warriors);
				}
				else {
					message.append("no");
				}
				message.append(" warriors");

				if (warriors != 0 && riders == 0) {
					message.append(" but ");
				}
				else {
					message.append(" and ");
				}
				
				if (riders != 0) {
					message.append(riders);
				}
				else {
					message.append("no");
				}
				message.append(" riders.");
				//return;
			}
			stage++;
		}
				
		for (Iterator it = location.getCharacters().iterator(); it.hasNext();) {
			midnight.Character character = (midnight.Character) it.next();
			if (stage == step) {
				message.append(capitalize(character.getFullTitle())).append(" commands ");
				Army warriors = character.getWarriors();
				Army riders = character.getRiders();
				
				message.append(warriors.toString());
				if ((warriors.getHowMany() == 0 && riders.getHowMany() == 0) ||
					(warriors.getHowMany() !=0 && riders.getHowMany() != 0)) {
					message.append(" and ");
				}
				else {
					message.append(" but ");
				}
				message.append(riders.toString());
				message.append(".");

				if (warriors != null && warriors.getHowMany() != 0) {
					message.append(" His warriors are ").append(warriors.getCondition()).append(". ");
				}
				if (riders != null && riders.getHowMany() != 0) {
					message.append(" His riders are ").append(riders.getCondition()).append(".");
				}

				//return;
			}
			stage++;
		}
		maxStep = stage;
		
		message.append("\n He thinks again....");
		text.setText(message.toString());
		text.paint(ig);
		g.drawImage(screen, 0, 0, getWidth(), getHeight(), null);
	}
	
	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();
		switch (key) {
			case KeyEvent.VK_ESCAPE:
			case 'e':
				step = 0;
				applet.mainScreen();
				break;
			case KeyEvent.VK_ENTER:
			case 'r':
				step++;
				if (step >= maxStep) {
					step = 0;
				}
				repaint();
				break;
			case 't':
				step = 0;
				applet.chooseAction();
				break;
			case 'u':
				step = 0;
				applet.night();
				break;
			default:
				return;
		}
	}
}
