/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight;

/**
 * Class representing game time. In LOM a game day consists of eight hours,
 * represented as sixteen half-hour units, with night equal to zero.
 */
public class Time {
	private int time;
	
	/**
	 * The constructor.
	 */
	public Time() {
		time = DAWN;
	}
	
	/**
	 * Constructor used when loading a saved game.
	 */
	protected Time(int time) {
		this.time = time;
	}
	
	/**
	 * Sets the current time to dawn.
	 */
	public void dawn() {
		time = DAWN;
	}
	
	/**
	 * Sets the current time to night.
	 */
	public void night() {
		time = NIGHT;
	}
	
	/**
	 * Returns a textual representation of the current time, given LOM-style,
	 * as the amount of hours left in the day and using "less than X hours"
	 * to represent internal half-hour increments.
	 */
	public String toString() {
		if (time == NIGHT) {
			return "It is night";
		}
		else if (time == DAWN) {
			return "It is dawn";
		}
		else {
			if (time % 2 == 1) {
				return "Less than " + (time / 2 + 1) + ((time < 3) ? " hour of the day remains" : " hours of the day remain");
			}
			return (time / 2) + ((time < 3) ? " hour of the day remains" : " hours of the day remain");
		}
	}
	
	/**
	 * Increases the amount of time left in the day by a given amount of
	 * half-hour units.
	 */
	public void increase(int increment) {
		time += increment;
		if (time > DAWN) time = DAWN;
	}
	
	/**
	 * Decreases the amount of time left in the day by a given amount of
	 * half-hour units.
	 */
	public void decrease(int decrement) {
		time -= decrement;
		if (time < NIGHT) time = NIGHT;
	}
	
	/**
	 * Returns true if the current time is night.
	 */
	public boolean isNight() {
		return time == NIGHT;
	}
	
	/**
	 * Returns true if the current time is dawn.
	 */
	public boolean isDawn() {
		return time == DAWN;
	}
	
	/**
	 * Returns the number of half-hour units left in the day.
	 */
	public int getTime() {
		return time;
	}
	
	private static int DAWN = 16;
	private static int NIGHT = 0;
}
