/*
 * Copyright 2003 Davor Cubranic
 *
 * This file is part of jLOM.
 *
 * jLOM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * jLOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with jLOM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package midnight.test;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.Iterator;

import midnight.Army;
import midnight.Character;
import midnight.Location;
import midnight.Map;
import midnight.Midnight;
import midnight.Race;
import midnight.Time;

/**
 * Class for testing the game engine aspects.
 */
public class Test {
	private static BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	private static String line;
	
	private static Midnight game;
	
	/**
	 * A simple mechanism for walking through the land.
	 * Keys 1-8 change the direction in which the character is looking,
	 * 0 walks by one space. Enter has to be pressed after every key.
	 */
	public static void main(String[] args) throws IOException {
		game = Midnight.getInstance();
		Map map = game.getMap();
		Character character = game.LUXOR;

//		for (int x = 0; x < 64; x++) {
//			System.out.println(x + " " + map.getLocation(x, 59));
//		}

		Location looking = map.getLookingTowards(character.getLocation(), character.getDirection());
		

		System.out.println(character.getFullTitle());
		System.out.print("He stands at " + character.getLocation());
		System.out.println(" looking " + character.getDirection() + " at " + looking);
		describeLocation(character.getLocation());
		printTimeOfDay(character.getTime());
		printCharacterEnergy(character);
		printCharacterCourage(character);
		while ((line = in.readLine()) != null) {
			if (line.equals("")) continue;
			char command = line.charAt(0);
			switch (command) {
				case 'q':
				case '0':
					if (character.canWalkForward()) {
						character.walkForward();
						looking = map.getLookingTowards(character.getLocation(), character.getDirection());
					}
					else {
						System.out.println("Cannot move");
					}
					break;
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
					character.setDirection(Map.Direction.getDirection(command - '1'));
					looking = map.getLookingTowards(character.getLocation(), character.getDirection());
					break;
				case 't':
					character = chooseAction(character);
					looking = map.getLookingTowards(character.getLocation(), character.getDirection());
					break;
				case 'c':
					character = game.LUXOR;
					break;
				case 'v':
					character = game.MORKIN;
					break;
				case 'b':
					character = game.CORLETH;
					break;
				case 'n':
					character = game.FARFLAME;
					break;
				case 'm':
					Character c = chooseCharacter();
					if (c != null) {
						character = c;
						looking = map.getLookingTowards(character.getLocation(), character.getDirection());
					}
					break;
				case 'u':
					Midnight.getInstance().night();
					break;
				case 's':
					int i = line.charAt(1) - '0';
					File f = new File("save" + i + ".lom");
					Midnight.getInstance().save(new DataOutputStream(new FileOutputStream(f)));
					break;
				case 'l':
					i = line.charAt(1) - '0';
					f = new File("save" + i + ".lom");
					if (f.canRead()) {
						Midnight.getInstance().load(new DataInputStream(new FileInputStream(f)));
						for (Iterator it = game.characters(); it.hasNext(); ) {
							c = (Character) it.next();
							if (character.getName().equals(c.getName())) {
								character = c;
								break;
							}
						}
					}
					break;
			}
			System.out.println(character.getFullTitle());
			System.out.print("He stands at " + character.getLocation());
			System.out.println(" looking " + character.getDirection() + " at " + looking);
			describeLocation(character.getLocation());
			printTimeOfDay(character.getTime());
			printCharacterEnergy(character);
			printCharacterCourage(character);
		}
	}
	
	private static void printTimeOfDay(Time time) {
		System.out.println(time);
	}
	
	private static void printCharacterEnergy(Character c) {
		System.out.println(c.getFullTitle() + " is " + c.getCondition());
		
		Army warriors = c.getWarriors();
		if (warriors.getHowMany() != 0) {
			System.out.println("His warriors are " + warriors.getCondition());
		}
		
		Army riders = c.getRiders();
		if (riders.getHowMany() != 0) {
			System.out.println("His riders are " + riders.getCondition());
		}
	}
	
	private static void printCharacterCourage(Character c) {
		System.out.println("Ice fear is " + c.getLocation().describeIceFear());
		System.out.println("He is " + c.getCourage() + ".");
	}
	
	private static Character chooseCharacter() {
		int i = 1;
		for (Iterator it = game.characters(); it.hasNext(); i++) {
			Character c = (Character) it.next();
			if (c.isRecruited()) {
				System.out.println(i + ". " + c.getName());
			}
		}
		
		int choice = -1;
		try {
			while ((line = in.readLine()) != null) {
				try {
					choice = Integer.parseInt(line);
					if (choice >= 0 && choice <= i) break;
				} catch (NumberFormatException ignored) {}
			}
		} catch (IOException ignored) {}
				
		i = 1;
		for (Iterator it = game.characters(); it.hasNext(); i++) {
			Character c = (Character) it.next();
			if (i == choice) return c;
		}
		
		return null;
	}	
	
	private static Character chooseAction(Character c) {
		HashSet validChoices = new HashSet();
		validChoices.add(new Integer(0));
		
		if (!c.isHidden()) {
			System.out.println("1. Seek");
			validChoices.add(new Integer(1));

			if (c != game.MORKIN) {
				System.out.println("2. Hide");
				validChoices.add(new Integer(2));
			}
		}
		else {
			System.out.println("2. Do not hide");
			validChoices.add(new Integer(2));
		}
		
		Location.Object object = c.getLocation().getObject();
		if (c.canFight()) {
			System.out.println("3. Fight the " + object);
			validChoices.add(new Integer(3));
		}
		
		Character recruiting = null;
		for (Iterator it = c.getLocation().getCharacters().iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			
			/*
			 * Original LOM limitation: if more than one character in location,
			 * can be recruited, will show the option only for the first one.
			 * It's never an issue in the game, though.
			 */
			if (c.canRecruit(character)) {
				System.out.println("4. Recruit " + character.getFullTitle());
				validChoices.add(new Integer(4));
				recruiting = character;
				break;
			}
		}
		
		Army guards = c.getLocation().getGuard();
		if (c.canRecruitMen()) {
			System.out.println("5. Recruit Men");
			validChoices.add(new Integer(5));
		}
		if (c.canStandOnGuard()) {
			System.out.println("6. Stand on guard");
			validChoices.add(new Integer(6));
		}
		
		/*
		 * LOM code also checks that FreeArmyPosInTable is >= 0x1d (what is it?)
		 */
		if (c.canAttack()) {
			System.out.println("7. To battle!");
			validChoices.add(new Integer(7));
		}
		
		int choice = -1;
		try {
			while ((line = in.readLine()) != null) {
				try {
					choice = Integer.parseInt(line);
					if (validChoices.contains(new Integer(choice))) break;
				} catch (NumberFormatException ignored) {}
			}
		} catch (IOException ignored) {}
		
		switch (choice) {
			case 1:
				Location.Object found = c.seek();
				System.out.println("You have found " + found);
				break;
			case 2:
				c.setHidden(!c.isHidden());
				break;
			case 3:
				c.fight();
				if (c.isAlive()) {
					System.out.println("He slew the " + object);
				}
				else {
					System.out.println("The " + object + " slew him.");
				}
				break;
			case 4:
				c.recruit(recruiting);
				return recruiting;
			case 5:
				c.recruitMen();
				break;
			case 6:
				c.standOnGuard();
				break;
			case 7:
				if (c.getLocation().getArmies().isEmpty() ||
					(guards == null || guards.getRace() != Race.FOUL)) {
					/* has to walk forward to get into the battle */
					c.walkForward();
				}
				break;
		}
		return c;
	}
	
	private static void describeLocation(Location location) {
		Army guard = location.getGuard();
		if (guard != null) {
			System.out.println(guard + " of the " + guard.getRace() + " guard the place.");
		}
		
		for (Iterator it = location.getCharacters().iterator(); it.hasNext();) {
			Character character = (Character) it.next();
			System.out.print(character.getFullTitle() + " has with him ");
			Army warriors = character.getWarriors();
			Army riders = character.getRiders();
			
			if (warriors.getHowMany() != 0 || riders.getHowMany() == 0) {
				System.out.print(warriors);
			}
			if ((warriors.getHowMany() == 0 && riders.getHowMany() == 0) ||
				(warriors.getHowMany() !=0 && riders.getHowMany() != 0)) {
				System.out.print(" and ");
			}
			if (riders.getHowMany() != 0 || warriors.getHowMany() == 0) {
				System.out.print(riders);
			}
			System.out.println(".");
		}
	}	
}
